SAL_FragLocator = {};

local pixStore = {};

local function GetDistance(p1, p2, mapName)
	local p1Pos = SAL_MapUtil.translateCoords(p1, mapName);
	local p2Pos = SAL_MapUtil.translateCoords(p2, mapName);
	
	local dx = p1Pos.x - p2Pos.x;
	local dy = p1Pos.y - p2Pos.y;
	
	return sqrt(dx*dx+dy*dy);
end

SAL_FragLocator.Init = function ()
	-- Use artifact data from gathermate if loaded
	if (IsAddOnLoaded("GatherMate2")) then
		SAL_FragLocator.gm2 = {};
		SAL_FragLocator.gm2.con = _G["GatherMate2"];
		SAL_FragLocator.gm2.map = SAL_FragLocator.gm2.con.mapData;
		SAL_FragLocator.gm2.loc = nil;
	else
		SAL_FragLocator.gm2 = nil;
	end
end

SAL_FragLocator.SyncFragmentLocation = function (map, loc, sync)
	if (sync == nil) then sync = false; end;
	if (SAL_FragLoc == nil) then
		SAL_FragLoc = {};
	end
	
	if (SAL_FragLoc[map] == nil) then
		SAL_FragLoc[map] = {};
	end
	
	local l = SAL_FragLoc[map];

	local closePos = nil;
	local closeDist = 999999999;
	for i,v in ipairs(l) do
		local dist = GetDistance(loc, v, map);
		if (dist < closeDist) then
			--print("Found closer: ", dist);
			closePos = v;
			closeDist = dist;
		end
	end
	
	if (closeDist < 5) then
		--print("Near other");
		if (sync) then
			if (loc.n > closePos.n) then
				closePos[1] = loc[1];
				closePos[2] = loc[2];
				closePos.n = loc.n;
			end
		else
			local x = closePos[1];
			local y = closePos[2];
			local n = closePos.n;
			closePos[1] = x * n / (n+1) + loc[1] / (n+1);
			closePos[2] = y * n / (n+1) + loc[2] / (n+1);
			closePos.n = closePos.n + 1;
		end
	else
		--print("Alone");
		l[#l + 1] = loc;
	end
	
	
	-- Reload gm2 data to display recently found location by forcing it to reload
	if (SAL_FragLocator.gm2 ~= nil) then
		SAL_FragLocator.gm2.loc = nil;
	end
	
	--print("Added loc in", mapName, "at", l[#l][1], l[#l][2]);
end

SAL_FragLocator.AddFragmentLocation = function ()
	local mapName = SAL_FixMapName(GetMapInfo());

	local myPos = {GetPlayerMapPosition("player")};
	myPos.n = 1;
	
	SAL_FragLocator.SyncFragmentLocation(mapName, myPos, false);
	--print("Added loc in", mapName, "at", l[#l][1], l[#l][2]);
end

SAL_FragLocator.GetCloseLocations = function ()
	local mapName = SAL_FixMapName(GetMapInfo());
	if (SAL_FragLocator.gm2 == nil) then
		-- Tracking using SAL	
		if (SAL_FragLoc == nil or SAL_FragLoc[mapName] == nil) then
			return {};
		end	
		
		return SAL_FragLoc[mapName];
	else
		-- Tracking using GatherMate2
		if (SAL_FragLocator.gm2.loc == nil or mapName ~= SAL_FragLocator.gm2.loc.mapName) then
			SAL_FragLocator.gm2.loc = SAL_FragLocator.GetGm2DataForZone();
		end
		
		return SAL_FragLocator.gm2.loc;
	end
end

SAL_FragLocator.ClearAllPix = function ()
	for i,v in ipairs(pixStore) do
		pixStore[i].active = false;
		pixStore[i].pix:Hide();
	end
end

local function GetNewPix()
	for i,v in ipairs(pixStore) do
		if not pixStore[i].active then
			pixStore[i].active = true;
			return pixStore[i].pix;
		end
	end

	local pix = { active=true, pix = SalFrame:CreateTexture(nil, "OVERLAY") };
	pix.pix:SetTexture("Interface/Archeology/Arch-Icon-Marker");
	pix.pix:SetWidth(15); 
	pix.pix:SetHeight(15);

	pixStore[#pixStore + 1] = pix;
	return pix.pix;
end

SAL_FragLocator.UpdatePositions = function ()
	SAL_FragLocator.ClearAllPix();
	if (SAL_MapUtil.hasCurrentMap() and SAL_Options.ShowFragments) then
		local l = SAL_FragLocator.GetCloseLocations();

		for i,v in ipairs(l) do
			local t = GetNewPix();
			local wPos = SAL_MapUtil.translateCoords(v);
			local sPos = SAL_calcPoint(wPos);
			t:SetPoint("CENTER", SalFrame, "BOTTOMLEFT", sPos.x, sPos.y);
			t:Show();
		end
	end
end

SAL_FragLocator.GetGm2DataForZone = function()
	local mapName = SAL_FixMapName(GetMapInfo());
	local mapId = SAL_FragLocator.gm2.map:MapAreaId(mapName);
	local gm2Lst = SAL_FragLocator.gm2.con.gmdbs["Archaeology"][mapId];	
	if (gm2Lst == nil) then
		return {mapName="nowhere_missingGM2Data"};
	end
	
	local salLst = {};
	for k,v in pairs(gm2Lst) do
		local x, y, lvl = SAL_FragLocator.gm2.map:DecodeLoc(k);
		local p = {x, y};
		p.n = 10;
		
		salLst[#salLst + 1] = p;
	end
	salLst.mapName = mapName;
	
	return salLst;
end

SAL_FragLocator.GetCount = function()
	local maps = 0;
	local locations = 0;
	local fragments = 0;
	for mapName,data in pairs(SAL_FragLoc) do 
		maps = maps + 1;
		for i,loc in ipairs(data) do 
			locations = locations + 1;
			fragments = fragments + loc.n;
		end;
	end;
	return maps, locations, fragments;
end