SAL_LineHandler = {};

local pixStore = {};

function SAL_LineHandler.ClearAllPix()
	for i,v in ipairs(pixStore) do
		pixStore[i].active = false;
		pixStore[i].pix:Hide();
	end
end

function SAL_LineHandler.GetNewPix()
	for i,v in ipairs(pixStore) do
		if not pixStore[i].active then
			pixStore[i].active = true;
			return pixStore[i].pix;
		end
	end

	local pix = { active=true, pix = SalFrame:CreateTexture(nil, "OVERLAY") };
	pixStore[#pixStore + 1] = pix;
	return pix.pix;
end

-- [[ Projection ]]
local GetCoordsForTransform = function(A, B, C, D, E, F)
	-- http://www.wowwiki.com/SetTexCoord_Transformations
	local det = A*E - B*D;
	local ULx, ULy, LLx, LLy, URx, URy, LRx, LRy;

	ULx, ULy = ( B*F - C*E ) / det, ( -(A*F) + C*D ) / det;
	LLx, LLy = ( -B + B*F - C*E ) / det, ( A - A*F + C*D ) / det;
	URx, URy = ( E + B*F - C*E ) / det, ( -D - A*F + C*D ) / det;
	LRx, LRy = ( E - B + B*F - C*E ) / det, ( -D + A -(A*F) + C*D ) / det;

	return ULx, ULy, LLx, LLy, URx, URy, LRx, LRy;
end;

function SAL_LineHandler.CreateLine(x, y, a, b, col)
	local ix = math.floor(x);
	local iy = math.floor(y);
	local ia = math.floor(a);
	local ib = math.floor(b);

	local cx, cy = (ix + ia)/2, (iy + ib)/2;
	local dx, dy = ix-ia, iy-ib;
	local dmax = math.max(math.abs(dx),math.abs(dy));
	local dr = math.sqrt(dx*dx + dy*dy);
	local scale = 1/dmax*32;
	local sinA, cosA = dy/dr*scale, dx/dr*scale;
	if dr == 0 then
		return nil;
	end

	local pix2 = SAL_LineHandler.GetNewPix();
	pix2:SetTexture("Interface\\AddOns\\Sal\\Artwork\\line.tga");
	pix2:SetVertexColor(col[1], col[2], col[3], col[4]);
	
	pix2:ClearAllPoints();

	pix2:SetPoint("CENTER", SalFrame, "BOTTOMLEFT", cx, cy);
	pix2:SetWidth(dmax); pix2:SetHeight(dmax);
	pix2:SetTexCoord(GetCoordsForTransform(
		cosA, sinA, -(cosA+sinA)/2+0.5,
		-sinA, cosA, -(-sinA+cosA)/2+0.5));
	pix2:Show();
end;
