local frame = CreateFrame("FRAME");
frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("PLAYER_LOGIN");
frame:RegisterEvent("VARIABLES_LOADED");
frame:RegisterEvent("UNIT_SPELLCAST_START");
frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
frame:RegisterEvent("PLAYER_REGEN_DISABLED");
frame:RegisterEvent("PLAYER_REGEN_ENABLED");

local SpellNames = {};
SpellNames["Survey"] = GetSpellInfo(80451);
SpellNames["SearchingForArtifacts"] = {};
SpellNames["SearchingForArtifacts"][1] = GetSpellInfo(74642);
SpellNames["SearchingForArtifacts"][2] = GetSpellInfo(74643);
SpellNames["SearchingForArtifacts"][3] = GetSpellInfo(74644);
SpellNames["SearchingForArtifacts"][4] = GetSpellInfo(74645);
SpellNames["SearchingForArtifacts"][5] = GetSpellInfo(74646);
SpellNames["SearchingForArtifacts"][6] = GetSpellInfo(74647);

local CombatHidden = false;
local LastSurvey = nil;
local LastMark = nil;

local traj =  {};

local drawOffset = nil;

---------------------------------------------------------------------

function SAL_UpdateVisibility()
	if (SAL_Visible and (not CombatHidden)) then
		SalFrame:Show();
		SalToolFrame:Show();
	else
		SalFrame:Hide();
		SalToolFrame:Hide();
	end
end

function SAL_Show()
	SAL_Visible = true;
	SAL_UpdateVisibility();
end

function SAL_Hide()
	SAL_Visible = false;
	SAL_UpdateVisibility();
end

----------------------- EVENT HANDLER ------------------------------
function frame:OnEvent(event, arg1, arg2, arg3, arg4)
	if event == "ADDON_LOADED" and arg1 == "SAL" then
		--print("S.A.L. loaded");

		if (SAL_Visible == nil) then
			SAL_Visible = true;
		end

		SAL_UpdateVisibility();
	elseif event == "PLAYER_LOGIN" then
		SAL_FragLocator.Init();
		SAL_SyncInit();
	elseif event == "VARIABLES_LOADED" then
		SAL_Options = SAL_OptionManager.Patch(SAL_Options);
	elseif event == "UNIT_SPELLCAST_START" then
		if (arg1 == "player" and arg2 == SpellNames["Survey"]) then
			if (SAL_Options.AutoShow) then
				SAL_Show();
			end
		end
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		if (arg1 == "player") then
			if (arg2 == SpellNames["SearchingForArtifacts"][1] or 
				arg2 == SpellNames["SearchingForArtifacts"][2] or
				arg2 == SpellNames["SearchingForArtifacts"][3] or
				arg2 == SpellNames["SearchingForArtifacts"][4] or
				arg2 == SpellNames["SearchingForArtifacts"][5] or
				arg2 == SpellNames["SearchingForArtifacts"][6]) then
				
				if (SAL_Options.AutoClear) then
					SAL_Clear();
				end
				if (SAL_Options.TrackFragments) then
					SAL_FragLocator.AddFragmentLocation();
				end
			elseif (arg2 == SpellNames["Survey"]) then
				LastSurvey = {time=GetTime(), pos=SAL_MapUtil.translateCoords({GetPlayerMapPosition("player")})};
			end 	
		end
	elseif event == "ZONE_CHANGED_NEW_AREA" then
		SAL_MapUtil.measuring = false;
	elseif event == "PLAYER_REGEN_DISABLED" then
		if (SAL_Options.CombatHide) then
			CombatHidden = true;
			SAL_UpdateVisibility();
		end
	elseif event == "PLAYER_REGEN_ENABLED" then
		CombatHidden = false;
		SAL_UpdateVisibility();
	end
end

frame:SetScript("OnEvent", frame.OnEvent);

function SAL_calcPoint(p)
	if (drawOffset == nil) then
		local l = SalFrame_MeMarker:GetLeft();
		local r = SalFrame_MeMarker:GetRight();
		local w = r - l;
		local t = SalFrame_MeMarker:GetTop();
		local b = SalFrame_MeMarker:GetBottom();
		local h = t - b;
		drawOffset = {x=(l + w / 2), y=(b + h / 2)};
	end
	local dawOrigo = SAL_MapUtil.translateCoords({GetPlayerMapPosition("player")});
	local x = (p.x - dawOrigo.x) * SAL_Options.drawScale + drawOffset.x;
	local y = UIParent:GetHeight() - ((p.y - dawOrigo.y) * SAL_Options.drawScale + drawOffset.y);
	return { x=x, y=y };
end

local function drawp(p1, p2, col)
	local p1_ = SAL_calcPoint(p1);
	local p2_ = SAL_calcPoint(p2);
	--print(format("drawing line from %i, %i to %i, %i", p1_.x, p1_.y, p2_.x, p2_.y));
	SAL_LineHandler.CreateLine(p1_.x, p1_.y, p2_.x, p2_.y, col);
end

local function draw()
	SAL_LineHandler.ClearAllPix();
	if (SAL_Visible) then
		for i,line in ipairs(traj) do	
			local alpha = 1 / (2 ^ (#traj - i));	-- Last is 1.0 there after 0.5 -> 0.25 -> 0.125 ...
			
			if (line.realCoords) then
				-- Green < 40 yd
				local l1 = 40;
				local p1 = {};		
				p1.x = line.origo.x + cos(line.v - 90) * l1;
				p1.y = line.origo.y + sin(line.v - 90) * l1;
				if (line.color == nil or line.color == "green") then
					drawp(line.origo, p1, {0, 1, 0, alpha});
				end

				-- Yellow < 80 yd
				local l2 = 80;
				local p2 = {};
				p2.x = line.origo.x + cos(line.v - 90) * l2;
				p2.y = line.origo.y + sin(line.v - 90) * l2;
				if (line.color == nil or line.color == "yellow") then
					drawp(p1, p2, {1, 1, 0, alpha});
				end

				-- Red > 80 yd
				local l3 = 250;
				local p3 = {};
				p3.x = line.origo.x + cos(line.v - 90) * l3;
				p3.y = line.origo.y + sin(line.v - 90) * l3;
				if (line.color == nil or line.color == "red") then
					drawp(p2, p3, {1, 0, 0, alpha});
				end

				if (line.ArcTxt ~= nil) then
					if (SAL_Options.ShowArc) then
						if (not line.ArcTxt:IsVisible()) then -- Restore texture if it has been removed
							line.ArcTxt = SAL_ArcHandler.GetArcPic(line);
						end
						local p = SAL_calcPoint(line.origo);
						line.ArcTxt:SetPoint("CENTER", SalFrame, "BOTTOMLEFT", p.x, p.y);
						line.ArcTxt:SetAlpha(alpha * SAL_Options.ArcOpac);
					else
						line.ArcTxt:Hide();
					end
				end
			else
				-- White
				local l = 250;
				local p = {};
				p.x = line.origo.x + cos(line.v) * l;
				p.y = line.origo.y + sin(line.v) * l;
				drawp(line.origo, p, {1, 1, 1, alpha});
			end
		end
		SalFrame_MeMarker:SetRotation(GetPlayerFacing());
		
		SAL_FragLocator.UpdatePositions();
		SAL_UpdateDigSiteBlobs();
	end
end

function SAL_Mark()
	local origo = SAL_MapUtil.translateCoords({GetPlayerMapPosition("player")});
	-- Check if last survey point should be used instead of current
--	if (LastSurvey ~= nil) then
--		local dx = origo.x - LastSurvey.pos.x;
--		local dy = origo.y - LastSurvey.pos.y;
--		if (LastSurvey.time + 10000 > GetTime() and sqrt(dx*dx+dy*dy) < 5) then
--			origo = LastSurvey.pos;
--		end
--	end
	local t = {};
	t.origo = origo;
	t.v = deg(-GetPlayerFacing());
	t.realCoords = SAL_MapUtil.hasCurrentMap();
	t.ArcTxt = SAL_ArcHandler.GetArcPic(t);
	t.time = GetTime();
	t.color = nil;
	traj[#traj + 1] = t;
	LastMark = t;
	LastSurvey = nil;
end

function SAL_ToggleMark()
	local myPos = SAL_MapUtil.translateCoords({GetPlayerMapPosition("player")});
	if (LastMark == nil or GetTime() > LastMark.time + 5000 or LastMark.origo.x ~= myPos.x or LastMark.origo.y ~= myPos.y) then
		SAL_Mark();
	end
	
	local t = traj[#traj];
	if (t.color == "red") then
		t.color = "green";
	elseif (t.color == "yellow") then
		t.color = "red";
	elseif (t.color == "green") then
		t.color = "yellow";
	else
		t.color = "green";
	end
	t.time = GetTime();
	t.ArcTxt.active = false;
	t.ArcTxt:Hide();
	t.ArcTxt = SAL_ArcHandler.GetArcPic(t);
end

function SAL_Clear(num)
	if (num == nil) then
		-- clear all
		traj = {};
	else
		-- remove last num
		local count = #traj;
		for i,v in ipairs(traj) do
			if (i > count - num) then
				traj[i] = nul;
			end
		end
	end
	LastMark = nil;
	SAL_ArcHandler.ClearArcPics();
end

-- Slash command
SLASH_SAL1 = "/sal";
SlashCmdList["SAL"] = function(msg)
	if (msg == nil or msg == "") then
		SAL_Visible = not SAL_Visible;
		SAL_UpdateVisibility();
	elseif (msg == "show") then
		Sal_Show();
	elseif (msg == "hide") then
		Sal_Hide();
	elseif (msg == "reset" or msg == "clear") then
		SAL_Clear();
	elseif (msg == "a") then
		SAL_Mark();
	elseif (msg == "measure") then
		print("Starting measure of current map, please move straight forward for a few seconds...");
		SAL_MapUtil.measuring = true;
	end
end

function SAL_OnUpdate(self, elapsed)
	if (SalFrame:IsShown()) then
		draw();
	end
	
	SAL_SyncTick();

	SAL_MapUtil.meausreTick();
end

local function desc(t)
	if (t == nil) then
		print("nil");
	elseif (type(t) == "table") then
		print(t);
		for k,v in pairs(t) do
			print(k, v);
		end
	elseif (type(t) == "function") then
		print(t);
	else
		print(type(t) .. ": " .. t);
	end
end
