local function clone(x)
	if (type(x) == "number") then
		return x;
	elseif (type(x) == "string") then
		return x;
	elseif (type(x) == "table") then
		local y = {};
		for k,v in pairs(x) do
			y[k] = clone(v);
		end
		return y;
	end
end

local receivedNodes = 0;
local sending = false;
local lastSent = 0;
local mapId = nil;
local mapQue = nil;

local function StartTransmit()
	receivedNodes = 0;
	sending = true;
	lastSent = 0;
	mapQue = {};
	for map,l in pairs(SAL_FragLoc) do
		local x={};
		x.name=map;
		x.data=l;
		mapQue[#mapQue+1] = x;
	end
	mapId = #mapQue;
end

function SAL_SetSyncPanel(x)
	--print("setting panel to ", x);
	SAL_CancelButton:Hide();
	SAL_OKButton:Hide();
	SAL_DenyButton:Hide();
	SAL_AcceptButton:Hide();
	SAL_AbortButton:Hide();
	SAL_DoneButton:Hide();
	
	SAL_ExportToTextBox:Hide();
	SAL_StatusBar:Hide();
	SAL_SenderCaptionText:Hide();
	SAL_SenderText:Hide();
	
	SAL_WaitingForReplyText:Hide();
	SAL_ExportToText:Hide();
	
	if (x == "OPEN") then
		SAL_CancelButton:Show();
		SAL_OKButton:Show();
		SAL_ExportToTextBox:Show();
		SAL_ExportToText:Show();
		
		SAL_SetCountTexts(SAL_FragLocator.GetCount());
	elseif (x == "SENTREQUEST") then
		SAL_AbortButton:Show();
		SAL_ExportToTextBox:Show();
		SAL_WaitingForReplyText:Show();
		SAL_ExportToText:Show();
	elseif (x == "GOTREQUEST") then
		SAL_DenyButton:Show();
		SAL_AcceptButton:Show();
		SAL_SenderCaptionText:Show();
		SAL_SenderText:Show();
	elseif (x == "SENDING") then
		SAL_AbortButton:Show();
		SAL_StatusBar:Show();
		SAL_SenderCaptionText:Show();
		SAL_SenderText:Show();
	elseif (x == "RECEIVING") then
		SAL_AbortButton:Show();
		SAL_StatusBar:Show();
		SAL_SenderCaptionText:Show();
		SAL_SenderText:Show();
	elseif (x == "DONE") then
		SAL_DoneButton:Show();
		--SAL_SenderCaptionText:Show();
		--SAL_SenderText:Show();
	end
end

function SAL_SetCountTexts(maps, nodes, finds)
	SAL_MapsCountText:SetText(maps);
	SAL_NodesCountText:SetText(nodes);
	SAL_FindingsCountText:SetText(finds);
end

----------------------- Buttons ----------------------------

function SAL_ButtonExport()
	local name = SAL_ExportToTextBox:GetText();
	local mapCount, nodeCount = SAL_FragLocator.GetCount();
	
	SAL_SyncComm:SyncRequest(name, mapCount, nodeCount, "SAL");
	
	SAL_SenderText:SetText(name);
	SAL_StatusBar:SetMinMaxValues(0, nodeCount);
	SAL_StatusBar:SetValue(0);
	receivedNodes = 0;
	
	SAL_SetSyncPanel("SENTREQUEST");
end

function SAL_ButtonDeny()
	SAL_SyncComm:SyncAbort(SAL_SyncComm.Current.otherSide, "Transaction canceled");
	SAL_SyncComm.Current = nil;
	SAL_SyncFrame:Hide();
end

function SAL_ButtonAccept()
	SAL_SyncComm:ReplySync(SAL_SyncComm.Current.otherSide, "OK");
	SAL_SetSyncPanel("RECEIVING");
end

function SAL_ButtonAbort()
	SAL_SyncComm:SyncAbort(SAL_SyncComm.Current.otherSide, "Transaction aborted");
	SAL_SyncComm.Current = nil;
	SAL_SetSyncPanel("DONE");
end

----------------------- Handlers ----------------------------

local function RequestHandler(sender, mapCount, nodeCount, source)
	SAL_SyncFrame:Show();
	
	SAL_SenderCaptionText:SetText("Incomming data from:");
	SAL_SenderText:SetText(sender);
	
	SAL_StatusBar:SetMinMaxValues(0, nodeCount);
	SAL_StatusBar:SetValue(0);
	receivedNodes = 0;
	
	SAL_SetCountTexts(mapCount, nodeCount, nil);
	
	SAL_SetSyncPanel("GOTREQUEST");
end

local function AbortHandler(msg)
	message("SAL - Transaction was aborted:\n" .. msg);
	SAL_SyncFrame:Hide();
end

local function ReplyHandler(success, msg)
	if (success) then
		SAL_SenderCaptionText:SetText("Sending data to:");
		SAL_SetSyncPanel("SENDING");
		StartTransmit();
	else
		message("Refused by receiver\n" .. msg);
	end
end

local function DataHandler(map, coords)
	receivedNodes = receivedNodes + #coords;
	SAL_StatusBar:SetValue(receivedNodes);
	
	--print("Received", #coords, "coords for", map);
	for i,loc in ipairs(coords) do
		--print(loc.n, loc[1], loc[2]);
		SAL_FragLocator.SyncFragmentLocation(map, loc, true);
	end
	
end

local function DataEndHandler()
	SAL_SetSyncPanel("DONE");
end

local function ErrorHandler(msg)
	SAL_SyncFrame:Hide();
	error(msg);
end

---------------------------------------------------

function SAL_SyncInit()
	SAL_SyncComm:Init(RequestHandler, AbortHandler, ReplyHandler, DataHandler, DataEndHandler, ErrorHandler);
	UIParent:HookScript("OnUpdate", SAL_SyncTick);
end

function SAL_SyncTick()
	if (sending and GetTime() > lastSent + 0.1) then
		local map = mapQue[mapId];
		if (map ~= nil) then
			mapQue[mapId] = nil;
			mapId = mapId - 1;
			--print("Sending ", map.name);
			SAL_SyncComm:Data(SAL_SyncComm.Current.otherSide, map.name, map.data);
			receivedNodes = receivedNodes + #map.data;
			lastSent = GetTime();
		else
			SAL_SyncComm:DataEnd(SAL_SyncComm.Current.otherSide);
			sending = false;
			--print("done sending");
			SAL_SetSyncPanel("DONE");
		end
	end

	SAL_StatusBar:SetValue(receivedNodes);
	SAL_SyncComm:Cleanup();
end
